-----------------------------------------
LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity csr_sipo_8b_inv is
generic (N:INTEGER:=4);
port(
		clk, clr, en, cir_shift: in std_logic; -- cir_shift '0' feed, '1' circular shift
		d: in std_logic_vector(7 downto 0);
		q: out std_logic_vector(8*N-1 downto 0)
);
end entity;


architecture arch of csr_sipo_8b_inv is
component reg_aclr_en_8bit 
port (clk:  in std_logic;
      clr, en:  in std_logic;
      d:    in std_logic_vector(7 downto 0);
      e:    out std_logic_vector(7 downto 0));
end component;

component inv_8b 
port (
      d:    in std_logic_vector(7 downto 0);
      e:    out std_logic_vector(7 downto 0));
end component;

signal dt: std_logic_vector(8*n-1 downto 0);
signal din_0, inv_dout_n: std_logic_vector(7 downto 0);

begin
com0: reg_aclr_en_8bit port map(clk, clr, en, din_0, e=>dt(7 downto 0));
din_0<=inv_dout_n when cir_shift='1' else d;

g1:for i in 1 to N-1 generate
 com1: reg_aclr_en_8bit port map(clk, clr, en, dt(8*i-1 downto 8*i-8), e=>dt(8*i+7 downto 8*i));
end generate;

com2: inv_8b port map(dt(8*n-1 downto 8*n-8), inv_dout_n); 
q<=dt;
end arch;
--------------------------------------------
----components
Library ieee;
Use ieee.std_logic_1164.all;

Entity fulladd is
	port ( Cin, x, y	: 	In	Std_logic;
		s, Cout		:	Out	Std_logic 
	);
end fulladd;

Architecture fulladd_logic of fulladd is
begin
	s <= x xor y xor Cin;
	Cout <= (x and y) or (Cin and x) or (Cin and y);
end fulladd_logic;

-- 8 bit Ripple-carry Adder
-- adder8.vhdl
Library ieee;
Use ieee.std_logic_1164.all;


Entity adder is
	port ( 
		a	:	In	Std_logic_vector(7 downto 0);
		b	:	In	Std_logic_vector(7 downto 0);	
		cin	:	In	Std_logic;
		c	:	Out	Std_logic_vector(7 downto 0)	
	);	
end adder;

Architecture adder_structure of adder is
	signal c1, c2, c3, c4	:		Std_logic;
	signal c5, c6, c7, cout	:		Std_logic;
        
component fulladd
port (	Cin, x, y	:	In	Std_logic;
	s, Cout		:	Out	Std_logic
	);
end component;

begin
	stage0: fulladd port map (cin, a(0), b(0), c(0), c1);
	stage1: fulladd port map (c1, a(1), b(1), c(1), c2);
	stage2: fulladd port map (c2, a(2), b(2), c(2), c3);
	stage3: fulladd port map (c3, a(3), b(3), c(3), c4);
	stage4: fulladd port map (c4, a(4), b(4), c(4), c5);
	stage5: fulladd port map (c5, a(5), b(5), c(5), c6);
	stage6: fulladd port map (c6, a(6), b(6), c(6), c7);
	stage7: fulladd port map (c7, a(7), b(7), c(7), Cout);
	
end adder_structure;
-------------------------------------
LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity inv_8b is
port (
      d:    in std_logic_vector(7 downto 0);
      e:    out std_logic_vector(7 downto 0));
end entity;

architecture arch of inv_8b is 
component adder 
	port ( 
		a	:	In	Std_logic_vector(7 downto 0);
		b	:	In	Std_logic_vector(7 downto 0);	
		cin	:	In	Std_logic;
		c	:	Out	Std_logic_vector(7 downto 0)	
	);	
end component;

signal d1,d2:std_logic_vector(7 downto 0);
begin
     d1<=d xor "11111111";
com0: adder port map(d1,"00000000",'1',e);
end arch;
-------------------------------------------------
LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity reg_aclr_en_8bit is
port (clk:  in std_logic;
      clr, en:  in std_logic;
      d:    in std_logic_vector(7 downto 0);
      e:    out std_logic_vector(7 downto 0));
end entity;

architecture arch of reg_aclr_en_8bit is 

begin
     process(clk, clr, en)
	  begin
             	if (clr='1') then
                 	e<="00000000";
		else
			
	    			if (clk'event and clk='1' and en='1' ) then             
		  			e<= d;		 
             			end if;
			
		end if;
	  end process;
end arch;