LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity control_unit is 

	generic(N:integer:=8);
	port (
			clk, clr: in std_logic; 
			cir_shift_input, cir_shift_output, en_csr_c1, en_csr_c2, en_csr_r2, en_csr_output, 
			clr_reg, add_c1, done: out std_logic
			);
			
end control_unit;

architecture arch of control_unit is

	type state_type is (state_reset, state_load, state_multi, state_add, state_output, state_done);
	signal present_state: state_type;
	signal count: integer range 0 to N-1;
	
	begin
		
		state: process(clr, clk, present_state)
		
		begin
		
			if clr = '1' then			
				present_state<=state_reset;
				done <= '0';
				en_csr_c1 <= '0';
				en_csr_c2 <= '0';
				en_csr_r2 <= '0';
				en_csr_output <= '0';
				cir_shift_input <= '0';
				cir_shift_output <= '0';
				clr_reg <= '1';
				add_c1 <= '0';
				
			elsif(clk'event and clk = '1') then
				
				case present_state is
				
					when state_reset =>
						present_state <= state_load;
						count <= 0;
						cir_shift_input <= '0';
						en_csr_c1 <= '1';
						en_csr_c2 <= '1';
						en_csr_r2 <= '1';
					
					when state_load =>
						if count < N-1 then
							count <= count + 1;
						elsif count = N-1 then
							present_state<=state_multi;
							cir_shift_input <= '1';
							en_csr_c1 <= '0';
							clr_reg <= '0';
							count <= 0;
						end if;
					
					when state_multi =>
						if count < N-1 then
							count <= count + 1;
						elsif count = N-1 then
							present_state <= state_add;
							en_csr_c1 <= '1';
							add_c1 <= '1';
							count <= 0;
						end if;
					
					when state_add => 
						if count = 0 then
							count <= count + 1;
							en_csr_output <= '1';
						elsif count = 1 then 
							present_state <= state_output;
							cir_shift_output <= '1';
							count <= 0;
						end if;
					
					when state_output =>
						if count < N-1 then
							count <= count + 1;
						elsif count = N-1 then 
							present_state <= state_done;
							done <= '1';
						end if;
					
					when state_done =>
						present_state <= state_done;
				
				end case;
				
			end if;
			
		end process;

end arch;
