library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity high_speed is
	
	generic (N : integer := 512);
	port ( c1: in std_logic_vector(7 downto 0);
			 c2: in std_logic_vector(7 downto 0);
			 r2,r3: in std_logic;
			 clk: in std_logic;
			 clr:	in std_logic;
			 output_8b: out std_logic_vector(7 downto 0);
			 output_1b:out std_logic;
			 done: out std_logic
			 --output_test: out std_logic_vector(8*N-1 downto 0)
			);
			
end entity;

architecture arch of high_speed is 

component HD 
port(a: in std_logic_vector(7 downto 0);
     carry_in: std_logic;
     b: out std_logic_vector(7 downto 0)
);
end component;
	
	component csr_siso_1b is
		generic (N:INTEGER:=4);
		port(
		clk, clr, en, cir_shift: in std_logic;-- cir_shift '0' feed, '1' circular shift
		d: in std_logic;
		q: out std_logic
			 );
	end component;
	
	component csr_sipo_8b_inv is
		generic (N:INTEGER:=4);
		port(
				clk, clr, en, cir_shift: in std_logic; -- cir_shift '0' feed, '1' circular shift
				d: in std_logic_vector(7 downto 0);
				q: out std_logic_vector(8*N-1 downto 0)
		);
	end component;
	
	component Full_Adder_8bit is
		port(x,y : in std_logic_vector(7 downto 0);
			  cin : in std_logic;
				sum : out std_logic_vector(7 downto 0);
				co : out std_logic
				);
	end component;
	
	component mux_2_to_1 is
	   port ( cin_0, cin_1: in std_logic_vector(7 downto 0);
				 s : in std_logic;
				 cout : out std_logic_vector(7 downto 0)
				 ); 			
	end component;
	
	component register_8_bit is
		port (clk: in std_logic;
				clr: in std_logic;
				cin: in std_logic_vector(7 downto 0);
				cout: out std_logic_vector(7 downto 0)
				);		
	end component;
	
	component sr_piso_wb is
		generic (N:INTEGER:=4;w:INTEGER:=8);
		port(
				clk, clr, en, sh: in std_logic; -- cir_shift '0' feed, '1' circular shift
				d: in std_logic_vector(w*n-1 downto 0);
				q: out std_logic_vector(w-1 downto 0)
			  );
	end component;
	
	component csr_sipo_8b is
		generic (N:INTEGER:=4);
		port(
				clk, clr, en, cir_shift: in std_logic; -- cir_shift '0' feed, '1' circular shift
				d: in std_logic_vector(7 downto 0);
				q: out std_logic_vector(8*N-1 downto 0)
				);
	end component;
	
	component control_unit is 
		generic(N:integer:=8);
		port (
				clk, clr: in std_logic; 
				cir_shift_input, cir_shift_output, en_csr_c1, en_csr_c2, en_csr_r2, en_csr_output, 
				clr_reg, add_c1, done: out std_logic
				);			
	end component;
	
		signal en_csr_c1_signal, en_csr_c2_signal, en_csr_r2_signal, en_csr_output_signal: std_logic;
		signal cir_shift_input_signal, cir_shift_output_signal: std_logic;
		signal clr_reg_signal, add_c1_signal:std_logic;
		signal csr_r2_out: std_logic;
		signal csr_c1_out, csr_c2_out: std_logic_vector(8*N-1 downto 0);
		signal multi_out, MUX_out, adder_out, reg_out: std_logic_vector(8*N-1 downto 0);
		signal csr_output_out: std_logic_vector(7 downto 0);

                signal HDout: std_logic_vector(7 downto 0);
	
	begin 
		
		-- control unit
		u0: control_unit
			 generic map(N => N)
			 port map(clk, clr, cir_shift_input_signal, cir_shift_output_signal, en_csr_c1_signal, 
						 en_csr_c2_signal, en_csr_r2_signal, en_csr_output_signal, clr_reg_signal, 
						 add_c1_signal, done);	
		
		-- input of c1
		-- In time order: din = c1(n-1), c1(n-2), c1(n-3)...c1(0)
		-- From most to least significant bit: output = [c1(n-1), c1(n-2), c1(n-3), ..., c1(0)] -> [c1(n-2), c1(n-3), c1(n-4), ..., c1(n-1)]
		u1: csr_sipo_8b
			 generic map(N => N)
			 port map(clk, clr, en_csr_c1_signal, cir_shift_input_signal, HDout, csr_c1_out);

                -- pre-addition of c1 and r3
                  u00: HD
                   port map(a=>c1, carry_in=>r3, b=>HDout );
		
		-- input of c2
		-- In time order: din = c2(n-1), c2(1), c2(2)...c2(0)
		-- From most to least significant bit: output = [c1(n-1), c1(n-2), c1(n-3), ..., c1(0)] -> [c1(n-2), c1(n-3), c1(n-4), ..., -c1(n-1)]
		u2: csr_sipo_8b_inv
			 generic map(N => N)
			 port map(clk, clr, en_csr_c2_signal, cir_shift_input_signal, c2, csr_c2_out);
			 
		-- input of r2
		-- In time order: din = r2(0), r2(1), r2(2)...r2(n-1)
		-- In time order: qout = r2(0), r2(1), r2(2)...r2(n-1)
		u3: csr_siso_1b
			 generic map(N => N)
			 port map(clk, clr, en_csr_r2_signal, cir_shift_input_signal, r2, csr_r2_out);
		
		-- multiplication
		g1: for i in 0 to N-1 generate 
			 comp1: mux_2_to_1 port map("00000000", csr_c2_out(8*i+7 downto 8*i), csr_r2_out, 
												 multi_out(8*i+7 downto 8*i));
			 end generate;
		
		-- MUX (add c1 or multi_out)
		g2: for i in 0 to N-1 generate 
			 comp2: mux_2_to_1 port map(multi_out(8*i+7 downto 8*i), csr_c1_out(8*i+7 downto 8*i), 
												 add_c1_signal, MUX_out(8*i+7 downto 8*i));
			 end generate;
			 
		-- accumulator
		-- adder 
		g3: for i in 0 to N-1 generate 
			 comp3:  Full_Adder_8bit port map(MUX_out(8*i+7 downto 8*i), reg_out(8*i+7 downto 8*i), '0',
														 adder_out(8*i+7 downto 8*i));
			 end generate;
		
		-- register
		g4: for i in 0 to N-1 generate 
			 comp4:  register_8_bit port map(clk, clr_reg_signal, adder_out(8*i+7 downto 8*i), 
														reg_out(8*i+7 downto 8*i));	
			 end generate;
			  
		-- output buffer
		-- output order: cout(n-1), cout(n-2), ..., cout(0)
		u4: sr_piso_wb generic map(N => N, w => 8) port map (clk, clr, en_csr_output_signal, 
																			  cir_shift_output_signal, reg_out,
																			  csr_output_out);
		
		-- output
		output_8b <= csr_output_out;
		output_1b <= csr_output_out(7) xor csr_output_out(6);

end arch;