LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity reg_aclr_en_nbit is
generic(N:INTEGER:=8);
port (clk:  in std_logic;
      clr, en:  in std_logic;
      d:    in std_logic_vector(n-1 downto 0);
      e:    out std_logic_vector(n-1 downto 0)
		);
end entity;

architecture arch of reg_aclr_en_nbit is 
signal zeros:std_logic_vector(n-1 downto 0):=(others=>'0');
begin
     process(clk, clr, en)
	  begin
             	if (clr='1') then
                 	e<=zeros;
		else
			
	    			if (clk'event and clk='1' and en='1' ) then             
		  			e<= d;		 
             			end if;
			
		end if;
	  end process;
end arch;
-----------------------------------------

LIBRARY ieee;               
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

entity sr_piso_wb is
generic (N:INTEGER:=4;w:INTEGER:=8);
port(
		clk, clr, en, sh: in std_logic; -- cir_shift '0' feed, '1' circular shift
		d: in std_logic_vector(w*n-1 downto 0);
		q: out std_logic_vector(w-1 downto 0)
);
end entity;

architecture arch of sr_piso_wb is
component reg_aclr_en_nbit
generic(N:INTEGER:=4);
port (clk:  in std_logic;
      clr, en:  in std_logic;
      d:    in std_logic_vector(w-1 downto 0);
      e:    out std_logic_vector(w-1 downto 0)
		);
end component;

signal dt,de: std_logic_vector(w*n-1 downto 0);
signal zeros:std_logic_vector(w-1 downto 0):=(others=>'0');

begin
unit0:  reg_aclr_en_nbit generic map (n=>w)
		port map(clk=>clk,
      clr=>clr, en=>en,
      d=>dt(w-1 downto 0),
      e=>de(w-1 downto 0)
		);
		
dt(w-1 downto 0)<=zeros when sh='1' else d(w-1 downto 0);
		
g1: for i in 1 to n-1 generate
	unit:  reg_aclr_en_nbit generic map (n=>w) 
		port map(clk=>clk,
      clr=>clr, en=>en,
      d=>dt(w*i+w-1 downto w*i),
      e=>de(w*i+w-1 downto w*i)
		);
		
	dt(w*i+w-1 downto w*i)<=de(w*i-1 downto w*i-w) when sh='1' else d(w*i+w-1 downto w*i);
end generate;
q<=de(w*n-1 downto w*n-w);
end arch;